package eventbasedexcelextractor;

/* ====================================================================
 Licensed to the Apache Software Foundation (ASF) under one or more
 contributor license agreements.  See the NOTICE file distributed with
 this work for additional information regarding copyright ownership.
 The ASF licenses this file to You under the Apache License, Version 2.0
 (the "License"); you may not use this file except in compliance with
 the License.  You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ==================================================================== */

import java.io.IOException;
import org.apache.poi.POIOLE2TextExtractor;
import org.apache.poi.hpsf.DocumentSummaryInformation;
import org.apache.poi.hpsf.SummaryInformation;
import org.apache.poi.hssf.eventusermodel.FormatTrackingHSSFListener;
import org.apache.poi.hssf.eventusermodel.HSSFEventFactory;
import org.apache.poi.hssf.eventusermodel.HSSFRequest;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

/**
 * A text extractor for Excel files, that is based on the hssf eventusermodel api. 
 * It will typically use less memory than {@link ExcelExtractor}, 
 * but may not provide the same richness of formatting. Returns the textual content of
 * the file, suitable for indexing by something like Lucene, but not really
 * intended for display to the user.
 * <p>
 * To turn an excel file into a CSV or similar, then see the XLS2CSVmra example
 * </p>
 * <link href=
 * "http://svn.apache.org/repos/asf/poi/trunk/src/examples/src/org/apache/poi/hssf/eventusermodel/examples/XLS2CSVmra.java"
 * > http://svn.apache.org/repos/asf/poi/trunk/src/examples/src/org/apache/poi/
 * hssf/eventusermodel/examples/XLS2CSVmra.java</link>
 */
public class EventBasedExcelExtractor extends POIOLE2TextExtractor {

	boolean includeSheetNames = true;

	// true if get the formula string, false if return result of formula
	boolean formulasNotResults = false;
	
	private DirectoryNode directoryNode;

	/**
	 * @deprecated Use {@link #EventBasedExcelExtractor(DirectoryNode)} instead
	 */
	@Deprecated
	@SuppressWarnings("unused")
	public EventBasedExcelExtractor(DirectoryNode dir, POIFSFileSystem fs) {
		this(dir);
	}

	/**
	 * constructor
	 * 
	 * @param directoryNode
	 */
	public EventBasedExcelExtractor(DirectoryNode directoryNode) {
		super(null);
		this.directoryNode = directoryNode;
	}

	/**
	 * constructor
	 * 
	 * @param poiFSFileSystem
	 */
	public EventBasedExcelExtractor(POIFSFileSystem poiFSFileSystem) {
		this(poiFSFileSystem.getRoot());
	}

	/**
	 * Return the underlying POIFS FileSystem of this document.
	 */
	@Override
	public POIFSFileSystem getFileSystem() {
		return directoryNode.getFileSystem();
	}

	/**
	 * Would return the document information metadata for the document, if supported 
	 */
	
	@Override
	public DocumentSummaryInformation getDocSummaryInformation() {
		throw new IllegalStateException("Metadata extraction not supported in streaming mode, please use ExcelExtractor");
	}

	/**
	 * Would return the summary information metadata for the document, if supported
	 */
	@Override
	public SummaryInformation getSummaryInformation() {
		throw new IllegalStateException("Metadata extraction not supported in streaming mode, please use ExcelExtractor");
	}

	/**
	 * @param true is sheet names should be included
	 */
	public void setIncludeSheetNames(boolean includeSheetNames) {
		this.includeSheetNames = includeSheetNames;
	}

	/**
	 * @param true if return the formula string, false if return result of formula
	 */
	public void setFormulasNotResults(boolean formulasNotResults) {
		this.formulasNotResults = formulasNotResults;
	}

	/**
	 * Retrieves the text contents of the file
	 */
	@Override
	public String getText() {
		String text = null;
		try {
			TextHSSFListener textHSSFListener = triggerExtraction(formulasNotResults, includeSheetNames);

			text = textHSSFListener.sheetNameStringBuffer.toString();
			if ( ! text.endsWith("\n")) {
				text = text + "\n";
			}
		} 
		catch (IOException e) {
			throw new RuntimeException(e);
		}

		return text;
	}

	/**
	 * 
	 * @return
	 * @throws IOException
	 */
	private TextHSSFListener triggerExtraction(
			boolean formulasNotResults, 
			boolean includeSheetNames) throws IOException {
		
		TextHSSFListener textListener = new TextHSSFListener(formulasNotResults, includeSheetNames);
		FormatTrackingHSSFListener formatTrackingHSSFListener = new FormatTrackingHSSFListener(textListener);
		textListener.formatTrackingHSSFListener = formatTrackingHSSFListener;

		// Register and process
		HSSFEventFactory factory = new HSSFEventFactory();
		HSSFRequest request = new HSSFRequest();
		request.addListenerForAllRecords(formatTrackingHSSFListener);

		factory.processWorkbookEvents(request, directoryNode);

		return textListener;
	}

}
